#include "base.h"

G_DEFINE_ABSTRACT_TYPE(BaseObject, base_object, G_TYPE_OBJECT)

/**
 * @brief Dispose the object
 */
static void base_object_dispose(GObject* object)
{
    g_debug("base_object_dispose");
    G_OBJECT_CLASS (base_object_parent_class)->dispose(object);
}

/**
 * @brief Class initialization
 */
static void base_object_class_init(BaseObjectClass *class)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS(class);
    class->to_string = NULL;
    gobject_class->dispose = base_object_dispose;
}

/**
 * @brief Object initialization
 */
static void base_object_init(BaseObject *self)
{
    (void) self;
}

/**
 * @brief Get the string representation of the object
 */
const gchar* base_object_to_string(gpointer object)
{
    g_return_val_if_fail(BASE_IS_OBJECT(object), NULL);
    BaseObject *self = BASE_OBJECT(object);
    BaseObjectClass *class = BASE_OBJECT_GET_CLASS(self);
    if (class->to_string)
    {
        g_debug("calling to_string");
        return class->to_string(self);
    }
    return "BaseObject";
}