#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <glib-unix.h>

#include "manager.h"
#include "configuration.h"

#define DEFALT_CONF_FILE "/etc/diya/daemon.conf"

static gchar **g_shell_argv;

static void help(const char* name)
{
    printf("Usage: %s [-c config_file]\n" ,name);
}

static gboolean restart(gpointer d)
{
    (void)d;
    gint i, fdlimit;

    fdlimit = (int)sysconf(_SC_OPEN_MAX);
    g_debug("reload: closing fd's %d to %d", STDERR_FILENO + 1, fdlimit);
    for (i = STDERR_FILENO + 1; i < fdlimit; i++)
    {
        fcntl(i, F_SETFD, FD_CLOEXEC);
    }
    g_debug("reload: exec: %s", g_shell_argv[0]);
    execvp(g_shell_argv[0], g_shell_argv);
    exit(1);
    return FALSE;
}

static gboolean quit(gpointer d)
{
    g_debug("Quit the daemon");
    session_manager_event_loop_stop((SessionManager*)d);
    return TRUE;
}

int main(int argc, char* const *argv)
{
    /**
     * Read command line arguments using getopt:
     * - if the argument is -h or --help print the help message
     * - if the argument is -c or --config set the configuration file
     */
    const gchar* config_file = DEFALT_CONF_FILE;
    int opt;
    while ((opt = getopt(argc, argv, "hc:")) != -1)
    {
        switch (opt)
        {
        case 'h':
            help(argv[0]);
            return 0;
        case 'c':
            config_file = optarg;
            break;
        default:
            help(argv[0]);
            return 1;
        }
    }
    g_shell_argv = g_malloc0(sizeof(gchar *) * (argc + 1));
    for (int i = 0; i < argc; i++)
    {
        g_shell_argv[i] = argv[i];
    }
    SessionManager* manager = session_manager_new(config_file);
    if (!manager)
    {
        g_error("Error creating the session manager");
        return -1;
    }
    g_unix_signal_add(SIGHUP, (GSourceFunc)restart, NULL);
    g_unix_signal_add(SIGINT,(GSourceFunc)quit,(void*)manager);
    DaemonConfiguration* conf = NULL;
    g_object_get(manager, "configuration", &conf, NULL);
    if(conf)
    {
        g_debug("Run daemon with: %s", base_object_to_string(conf));
    }
    session_manager_event_loop_start(manager);
    g_debug("Release the manager");
    g_object_unref(manager);
    return 0;
}
